import { Directive, ElementRef, HostBinding, Input, Renderer } from '@angular/core';
import { Config } from '../../config/config';
import { Ion } from '../ion';
export class Icon extends Ion {
    constructor(config, elementRef, renderer) {
        super(config, elementRef, renderer);
        this._name = '';
        this._ios = '';
        this._md = '';
        this._css = '';
        this._hidden = false;
        this.mode = config.get('mode');
        this._iconMode = config.get('iconMode');
    }
    get color() {
        return this._color;
    }
    set color(value) {
        this._setColor('icon', value);
    }
    set mode(val) {
        this._setMode('icon', val);
    }
    ngOnDestroy() {
        if (this._css) {
            this.setElementClass(this._css, false);
        }
    }
    get name() {
        return this._name;
    }
    set name(val) {
        if (!(/^md-|^ios-|^logo-/.test(val))) {
            val = this._iconMode + '-' + val;
        }
        this._name = val;
        this.update();
    }
    get ios() {
        return this._ios;
    }
    set ios(val) {
        this._ios = val;
        this.update();
    }
    get md() {
        return this._md;
    }
    set md(val) {
        this._md = val;
        this.update();
    }
    get isActive() {
        return (this._isActive === undefined || this._isActive === true || this._isActive === 'true');
    }
    set isActive(val) {
        this._isActive = val;
        this.update();
    }
    update() {
        let css = 'ion-';
        this._hidden = (this._name === null);
        if (this._ios && this._iconMode === 'ios') {
            css += this._ios;
        }
        else if (this._md && this._iconMode === 'md') {
            css += this._md;
        }
        else {
            css += this._name;
        }
        if (this._iconMode === 'ios' && !this.isActive && css.indexOf('logo') < 0) {
            css += '-outline';
        }
        if (this._css !== css) {
            if (this._css) {
                this.setElementClass(this._css, false);
            }
            this._css = css;
            this.setElementClass(css, true);
            this.setElementAttribute('aria-label', css.replace('ion-', '').replace('ios-', '').replace('md-', '').replace('-', ' '));
        }
    }
}
Icon.decorators = [
    { type: Directive, args: [{
                selector: 'ion-icon',
                host: {
                    'role': 'img'
                }
            },] },
];
Icon.ctorParameters = [
    { type: Config, },
    { type: ElementRef, },
    { type: Renderer, },
];
Icon.propDecorators = {
    'color': [{ type: Input },],
    'mode': [{ type: Input },],
    'name': [{ type: Input },],
    'ios': [{ type: Input },],
    'md': [{ type: Input },],
    'isActive': [{ type: Input },],
    '_hidden': [{ type: HostBinding, args: ['class.hide',] },],
};
